<?php
/*
 * zfs.inc
 *
 * part of pfSense (https://www.pfsense.org)
 * Copyright (c) 2021-2022 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require_once('vendor/autoload.php');

use Netgate\Services\ZFS\LibZFS;

use Netgate\Services\ZFS\Pool;
use Netgate\Services\ZFS\VDev;
use Netgate\Services\ZFS\VDevType;
use Netgate\Services\ZFS\ZFS;

use Nette\Utils\Arrays;
use Nette\Utils\Html;
use Nette\Utils\Strings;

$zfs_title = gettext('ZFS');

function zfs_compose_widget_table() {
	$table = Html::el(null);

	$table->addHtml(zfs_compose_widget_header());

	$table->addHtml(zfs_compose_widget_body());

	return $table->toHtml();
}

function zfs_compose_widget_header() {
	$thead = Html::el('thead');

	$headers = array_keys(zfs_Widget_pool_row());

	foreach ($headers as $header) {
		$thead->addHtml(
			Html::el('th')
				->setText($header));
	}

	return $thead->toHtml();
}

function zfs_compose_widget_body() {
	$zfs = new Zfs();

	$tbody = Html::el('tbody');

	if ($zfs->hasPools()) {
		foreach ($zfs->getPools() as $pool) {
			$tbody->addHtml(zfs_compose_pool_row($pool));

			$tbody->addHtml(zfs_compose_vdev_header($pool));

			$tbody->addHtml(zfs_compose_vdev_rows($pool));
		}

		return $tbody->toHtml();
	}

	$tbody->addHtml(
		Html::el('tr')
			->addHtml(
				Html::el('td')
					->setAttribute('colspan', '5')
					->setText(gettext('No ZFS pools are configured.'))));

	return $tbody->toHtml();
}

function zfs_compose_pool_row(Pool $pool) {
	$tr = Html::el('tr');

	$tr->setAttribute('class', $pool->getHtmlClass('treegrid-'));

	$values = Arrays::invoke(zfs_Widget_pool_row(), $pool);

	foreach ($values as $value) {
		$tr->addHtml($value);
	}

	return $tr;
}

function zfs_compose_vdev_header(Pool $pool) {
	$tr = Html::el('tr');

	$tr->setAttribute('class', $pool->getHtmlClass('treegrid-parent-'));

	$headers = array_keys(zfs_widget_vdev_row());

	foreach ($headers as $header) {
		$tr->addHtml(
			Html::el('td')
				->setText($header)
				->style('font-weight', 'bold'));
	}

	return $tr;
}

function zfs_compose_vdev_rows(Pool $pool) {
	$rows = Html::el(null);

	$icon_class = array(
			'cache'		=> 'fa fa-tachometer',
			'data'		=> 'fa fa-database',
			'dedup'		=> 'fa fa-project-diagram',
			'log'		=> 'fa fa-clipboard',
			'spare'		=> 'fa fa-life-bouy',
			'special'	=> 'fa fa-tags');

	foreach ($pool->getVdevTypes() as $type) {
		foreach ($type->getFlattenedVdevs() as $vdev) {
			$tr = Html::el('tr');

			$tr->appendAttribute('class', $vdev->getHtmlClass('treegrid-'));

			$tr->appendAttribute('class', $vdev->getParentHtmlClass('treegrid-parent-'));

			$values = Arrays::invoke(zfs_widget_vdev_row(), $vdev, $icon_class[$type->getType()], $type->getType());

			foreach ($values as $value) {
				$tr->addHtml($value);
			}

			$rows->addHtml($tr);
		}
	}

	return $rows;
}

function zfs_Widget_pool_row() {

	return array(
		gettext('Name') => function(Pool $p) { 
						$td = Html::el('td');

						$td->setHtml(zfs_compose_pool_health_icon($p));

						return $td;},

		gettext('Health') => function (Pool $p) {
						$td = Html::el('td');

						$td->setText($p->getProperty(['properties', 'health', 'value']));

						return $td;},

		gettext('Size') => function (Pool $p) {
						$td = Html::el('td');

						$td->setText($p->getProperty(['properties', 'size', 'value']));

						return $td;},

		gettext('Allocated') => function (Pool $p) {
						$td = Html::el('td');

						$td->setText($p->getProperty(['properties', 'allocated', 'value']));

						return $td;},

		gettext('Free') => function (Pool $p) {
						$td = Html::el('td');

						$td->setText($p->getProperty(['properties', 'free', 'value']));

						return $td;});
}

function zfs_widget_vdev_row() {
	return array(
		gettext('VDEV') => function (Vdev $v, $icon, $type) {
						$td = Html::el('td');
						
						$text = $v->getTypeFriendly();
						
						/**
						 * We only want rows with a VDevType as parent to have an icon
						 */
						if ($v->getParent() instanceof VDevType) {							
							$text = zfs_compose_icon_text($v->getTypeFriendly(), $icon, $type);
						}

						$td->setText($text);

						return $td;},

		gettext('Health') => function (Vdev $v, $icon, $type) {
						$td = Html::el('td');
	
						$td->setText($v->getProperty('status'));

						return $td;},

		gettext('Read Errors') => function (Vdev $v, $icon, $type) {
						$td = Html::el('td');

						$td->setText($v->getProperty(['stats', 'read_errors']));

						return $td;},

		gettext('Write Errors') => function (Vdev $v, $icon, $type) {
						$td = Html::el('td');

						$td->setText($v->getProperty(['stats', 'write_errors']));

						return $td;},

		gettext('Checksum Errors') => function (Vdev $v, $icon, $type) {
						$td = Html::el('td');

						$td->setText($v->getProperty(['stats', 'checksum_errors']));

						return $td;});
}

function zfs_compose_pool_health_icon(Pool $pool) {
	return zfs_compose_icon_text($pool->getName(), zfs_pool_health_icon_class($pool));
}

function zfs_compose_icon_text($text, $icon_class, $title = null) {
	$i = Html::el('i');

	$i->setAttribute('class', $icon_class);

	$i->setAttribute('title', is_null($title) ? $text : $title);

	$i->style('vertical-align', 'middle');

	$i->style('padding-right', '5px');

	return Html::el(null)->addHtml($i)->addText($text);
}

function zfs_pool_health_icon_class(Pool $pool) {
	$health = $pool->getProperty(['properties', 'health', 'value']);

	switch ($health) {
		case 'ONLINE':
			return 'fa fa-check-circle text-success';
			break;

		case 'DEGRADED':
			return 'fa fa-exclamation-circle text-warning';
			break;

		case 'FAULTED':

		case 'OFFLINE':

		case 'UNAVAIL':

		case 'REMOVED':
			return 'fa fa-times-circle text-danger';
			break;

		default:
			return 'fa fa-question-circle text-warning';
			break;
	}
}

function zfs_cache_invalidate($force, $chancef = 0.1) {
	if (((rand() / getrandmax()) < $chancef) || $force)  {
		(new Libzfs)->flushCache();
	}
}

?>
